#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define SERIAL_PORT     "/dev/ttyAM1"

void portAllData8bit(int fd, int numGetData, int id);
void port1Data8bit(int fd, int numGetData, int id, int port);
void portAllData12bit(int fd, int numGetData, int id);
void port1Data12bit(int fd, int numGetData, int id, int port);

int main(){
  int fd;
  fd = open(SERIAL_PORT, O_RDWR | O_NOCTTY);
  if(fd < 0){
    printf("%s doesn't open it\n",SERIAL_PORT);
    return -1;
  }

  struct termios oldtio, newtio;
  tcgetattr(fd, &oldtio);
  newtio = oldtio;

  cfsetspeed(&newtio, B9600);

  /* {[h̐ݒ */
  int id, port, resolution;
  printf("ID of AGB65-ADC[default:120]");
  scanf("%d", &id);
  printf("Sensor Port on AGB65-ADC[1-16, 0:ALL, default:0]");
  scanf("%d", &port);
  printf("Resolution of data[0:(8bit) or 1:(16bit), default:0]");
  scanf("%d", &resolution);

  /* s */
  int numGetData;
  printf("Repeat time to get sensor data[default:5]");
  scanf("%d", &numGetData);

  /* f[^ǂݍݐݒ */
  if(port == 0 && resolution == 0){
    /* S|[gǂݎ & f[^𑜓x 8bit */
    portAllData8bit(fd, numGetData, id);
  } else if(port != 0 && resolution == 0){
    /* 1|[gǂݎ & f[^𑜓x 8bit */
    port1Data8bit(fd, numGetData, id, port);
  } else if(port == 0 && resolution == 1){
    /* S|[gǂݎ & f[^𑜓x 12bit */
    portAllData12bit(fd, numGetData, id);
  } else if(port != 0 && resolution == 1){
    /* 1|[gǂݎ & f[^𑜓x 12bit */
    port1Data12bit(fd, numGetData, id, port);
  }

  tcsetattr(fd, TCSANOW, &oldtio);
  close(fd);
  return 0;
}

void portAllData8bit(int fd, int numGetData, int id){
  int uWait, retry;
  printf("Waiting time from sending command to getting data\n");
  printf("[about 25000uSec on B9600, about 2600uSec on B115200]:");
  scanf("%d", &uWait);
  printf("Retry number[2-4, related with waiting time]:");
  scanf("%d", &retry);

  int i, j, k;
  unsigned char command[4] = {255, id, 1, 1};
  unsigned char data[20];
  
  for(i=1 ; i<=numGetData ; i++){
    printf("Getting data : %d\n", i);

    /* f[^Mv */
    write(fd, command, sizeof(command));
    for(j=0 ; j<retry ; j++){
      usleep(uWait);
      read(fd, data, sizeof(data));
      /* data check */
      if(data[0] == 255 && data[1] == id && data[2] == 17 && data[3] == 1){
        printf("Retry number: %d\n", j);
        for(k=4 ; k<20 ; k++){
          printf("port%2d : data=%d\n", j-3, data[j]);
        }
      }
    }
  }
  
  return;
}

void port1Data8bit(int fd, int numGetData, int id, int port){
  int uWait, retry;
  printf("Waiting time from sending command to getting data\n");
  printf("[about 12000uSec on B9600, about 1400uSec on B115200]:");
  scanf("%d", &uWait);
  printf("Retry number[2-4, related with waiting time]:");
  scanf("%d", &retry);

  int i, j;
  unsigned char command[5] = {255, id, 2, 2, port};
  unsigned char data[6];
  
  for(i=1 ; i<=numGetData ; i++){
    printf("Getting data : %d\n", i);

    /* f[^Mv */
    write(fd, command, sizeof(command));
    for(j=0 ; j<retry ; j++){
      usleep(uWait);
      read(fd, data, sizeof(data));
      /* data check */
      if(data[0] == 255 && data[1] == id && data[2] == 3 && data[3] == 2 && data[4] == port){
        printf("Retry number: %d\n", j);
        printf("port%2d : data=%d\n", port, data[5]);
      }
    }
  }
  
  return;
}

void portAllData12bit(int fd, int numGetData, int id){
  int uWait, retry;
  printf("Waiting time from sending command to getting data\n");
  printf("[about 42000uSec on B9600, about 4000uSec on B115200]:");
  scanf("%d", &uWait);
  printf("Retry number[2-4, related with waiting time]:");
  scanf("%d", &retry);

  int i, j, k, tmp;
  unsigned char command[4] = {255, id, 1, 11};
  unsigned char data[36];
  
  for(i=1 ; i<=numGetData ; i++){
    printf("Getting data : %d\n", i);

    /* f[^Mv */
    write(fd, command, sizeof(command));
    for(j=0 ; j<retry ; j++){
      usleep(uWait);
      read(fd, data, sizeof(data));
      /* data check */
      if(data[0] == 255 && data[1] == id && data[2] == 33 && data[3] == 11){
        printf("Retry number: %d\n", j);
        for(k=4 ; k<36 ; k+=2){ /* f[^[Pi_H][Pi_L]ƏʁEʂ̏2oCgP */
          tmp = (data[k] << 8);
          tmp += data[k+1];
          printf("port%2d : High=%d : Low=%d : data=%d\n", k/2-1, data[k], data[k+1], tmp);
        }
      }
    }
  }
  
  return;
}

void port1Data12bit(int fd, int numGetData, int id, int port){
  int uWait, retry;
  printf("Waiting time from sending command to getting data\n");
  printf("[about 13000uSec on B9600, about 1500uSec on B115200]:");
  scanf("%d", &uWait);
  printf("Retry number[2-4, related with waiting time]:");
  scanf("%d", &retry);

  int i, j, tmp;
  unsigned char command[5] = {255, id, 2, 12, port};
  unsigned char data[6];
  
  for(i=1 ; i<=numGetData ; i++){
    printf("Getting data : %d\n", i);

    /* f[^Mv */
    write(fd, command, sizeof(command));
    for(j=0 ; j<retry ; j++){
      usleep(uWait);
      read(fd, data, sizeof(data));
      /* data check */
      if(data[0] == 255 && data[1] == id && data[2] == 4 && data[3] == 12){
        printf("Retry number: %d\n", j);
        /* f[^[Pi_H][Pi_L]ƏʁEʂ̏2oCgP */
        tmp = (data[4] << 8);
        tmp += data[5];
        printf("port%2d : High=%d : Low=%d : data=%d\n", port, data[4], data[5], tmp);
      }
    }
  }
  
  return;
}

